@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2017  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2017  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -395,7 +381,6 @@ static void mbim_netreg_remove(struct ofono_netreg *netreg)
 }
 
 static const struct ofono_netreg_driver driver = {
-	.name				= "mbim",
 	.probe				= mbim_netreg_probe,
 	.remove				= mbim_netreg_remove,
 	.registration_status		= mbim_registration_status,
@@ -404,12 +389,4 @@ static const struct ofono_netreg_driver driver = {
 	.strength			= mbim_signal_strength,
 };
 
-void mbim_netreg_init(void)
-{
-	ofono_netreg_driver_register(&driver);
-}
-
-void mbim_netreg_exit(void)
-{
-	ofono_netreg_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(netreg, mbim, &driver)
