@@ -15,6 +15,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <net/if.h>
+#include <sys/stat.h>
 
 #define OFONO_API_SUBJECT_TO_CHANGE
 #include <ofono/plugin.h>
@@ -45,6 +46,8 @@
 #include <drivers/qmimodem/dms.h>
 #include <drivers/qmimodem/wda.h>
 #include <drivers/qmimodem/util.h>
+#include <drivers/qmimodem/common.h>
+#include "src/rmnet.h"
 
 #define GOBI_DMS	(1 << 0)
 #define GOBI_NAS	(1 << 1)
@@ -56,6 +59,19 @@
 #define GOBI_WDA	(1 << 7)
 
 #define MAX_CONTEXTS 4
+#define DEFAULT_MTU 1400
+#define RMNET_MAX_MTU 16384
+#define DEFAULT_DL_DATAGRAMS 32
+
+#define QMI_WWAN_RAW_IP "/sys/class/net/%s/qmi/raw_ip"
+#define QMI_WWAN_PASS_THROUGH "/sys/class/net/%s/qmi/pass_through"
+
+enum wda_data_format {
+	WDA_DATA_FORMAT_UNKNOWN = 0,
+	WDA_DATA_FORMAT_RMNET_QMAP5,
+	WDA_DATA_FORMAT_RAW_IP,
+	WDA_DATA_FORMAT_802_3,	/* Last, most compatible legacy fallback */
+};
 
 struct service_request {
 	struct qmi_service **member;
@@ -76,20 +92,23 @@ struct gobi_data {
 		struct qmi_service *wds_ipv4;
 		struct qmi_service *wds_ipv6;
 	} context_services[MAX_CONTEXTS];
+	struct rmnet_ifinfo rmnet_interfaces[MAX_CONTEXTS];
+	uint8_t n_premux;
+	int rmnet_id;
 	struct service_request service_requests[8 + MAX_CONTEXTS * 2];
 	int cur_service_request;
 	int num_service_requests;
 	unsigned long features;
 	unsigned int discover_attempts;
-	uint8_t n_premux;
 	uint8_t oper_mode;
 	int main_net_ifindex;
 	char main_net_name[IFNAMSIZ];
+	uint8_t interface_number;
 	uint32_t max_aggregation_size;
 	uint32_t set_powered_id;
-	bool using_mux : 1;
-	bool using_qmi_wwan : 1;
-	bool using_qmi_wwan_q : 1;
+	uint32_t set_mtu_id;
+	enum wda_data_format data_format;
+	bool no_pass_through : 1;
 };
 
 static void gobi_debug(const char *str, void *user_data)
@@ -106,32 +125,103 @@ static void gobi_io_debug(const char *str, void *user_data)
 	ofono_debug("%s%s", prefix, str);
 }
 
+static bool wda_get_data_format(struct gobi_data *data,
+				struct qmi_wda_data_format *out_format)
+{
+	if (data->data_format == WDA_DATA_FORMAT_UNKNOWN ||
+			data->data_format > WDA_DATA_FORMAT_802_3)
+		return false;
+
+	memset(out_format, 0, sizeof(struct qmi_wda_data_format));
+
+	if (data->data_format == WDA_DATA_FORMAT_802_3) {
+		out_format->ll_protocol = QMI_WDA_DATA_LINK_PROTOCOL_802_3;
+		return true;
+	}
+
+	/* For everything except 802.3, use RAW_IP */
+	out_format->ll_protocol = QMI_WDA_DATA_LINK_PROTOCOL_RAW_IP;
+
+	if (data->data_format == WDA_DATA_FORMAT_RAW_IP)
+		goto done;
+
+	out_format->dl_max_datagrams = DEFAULT_DL_DATAGRAMS;
+	out_format->dl_max_size = data->max_aggregation_size;
+
+	if (data->data_format == WDA_DATA_FORMAT_RMNET_QMAP5) {
+		out_format->ul_aggregation_protocol =
+					QMI_WDA_AGGREGATION_PROTOCOL_QMAPV5;
+		out_format->dl_aggregation_protocol =
+					QMI_WDA_AGGREGATION_PROTOCOL_QMAPV5;
+	}
+done:
+	return true;
+}
+
+static int wda_data_format_to_mtu(enum wda_data_format format, uint32_t *out_mtu)
+{
+	uint32_t mtu;
+
+	switch (format) {
+	case WDA_DATA_FORMAT_802_3:
+	case WDA_DATA_FORMAT_RAW_IP:
+		mtu = DEFAULT_MTU;
+		break;
+	case WDA_DATA_FORMAT_RMNET_QMAP5:
+		mtu = RMNET_MAX_MTU;
+		break;
+	default:
+		return -ENOTSUP;
+	}
+
+	if (out_mtu)
+		*out_mtu = mtu;
+
+	return 0;
+}
+
+static int qmi_wwan_set_raw_ip(const char *interface, char value)
+{
+	return l_sysctl_set_char(value, QMI_WWAN_RAW_IP, interface);
+}
+
+static int qmi_wwan_set_pass_through(const char *interface, char value)
+{
+	return l_sysctl_set_char(value, QMI_WWAN_PASS_THROUGH, interface);
+}
+
 /*
  * Probe the modem.  The following modem properties are expected to be set
  * in order to initialize the driver properly:
  *
  * NetworkInterface
- *   The string that contains the 'main' network device.  This can be
- *   "rmnet_ipa" on SoC systems, or "wwan0" for upstream linux systems.
+ *   The string that contains the 'main' network device.  This is typically
+ *   'wwanX' on upstream linux systems.
  *
  * NetworkInterfaceIndex
  *   The index of the main interface given by NetworkInterface
  *
+ * InterfaceNumber
+ *   The USB interface number of the network interface
+ *
  * NetworkInterfaceKernelDriver
- *   The kernel driver that is being used by the main network device.  Certain
- *   drivers such as 'qmi_wwan' or 'qmi_wwan_q' are treated specifically.
+ *   The kernel driver that is being used by the main network device.  Only
+ *   'qmi_wwan' is supported.
  *
  * Bus
- *   The bus of the modem.  Values can be "usb", "embedded", or "pci"
+ *   The bus of the modem.  Values can be "usb"
  */
 static int gobi_probe(struct ofono_modem *modem)
 {
 	struct gobi_data *data;
 	const char *if_driver;
 	const char *ifname;
+	uint8_t interface_number;
 	int ifindex;
 	const char *bus;
-	int n_premux;
+	struct stat st;
+	_auto_(l_free) char *pass_through = NULL;
+	bool no_pass_through = false;
 
 	DBG("%p", modem);
 
@@ -140,32 +230,38 @@ static int gobi_probe(struct ofono_modem *modem)
 	ifname = ofono_modem_get_string(modem, "NetworkInterface");
 	ifindex = ofono_modem_get_integer(modem, "NetworkInterfaceIndex");
 	bus = ofono_modem_get_string(modem, "Bus");
-	n_premux = ofono_modem_get_integer(modem, "NumPremuxInterfaces");
 
 	DBG("net: %s[%s](%d) %s", ifname, if_driver, ifindex, bus);
 
-	if (!if_driver || !ifname || !ifindex || !bus || n_premux < 0)
+	if (!if_driver || !ifname || !ifindex || !bus)
 		return -EPROTO;
 
-	data = l_new(struct gobi_data, 1);
+	if (!L_IN_STRSET(if_driver, "qmi_wwan"))
+		return -ENOTSUP;
 
-	if (!strcmp(if_driver, "qmi_wwan_q"))
-		data->using_qmi_wwan_q = true;
-	else if (!strcmp(if_driver, "qmi_wwan"))
-		data->using_qmi_wwan = true;
+	if (!L_IN_STRSET(bus, "usb"))
+		return -ENOTSUP;
 
-	if (n_premux > MAX_CONTEXTS) {
-		l_warn("NumPremuxInterfaces > %d, limiting to %d",
-				MAX_CONTEXTS, MAX_CONTEXTS);
-		n_premux = MAX_CONTEXTS;
+	if (l_safe_atox8(ofono_modem_get_string(modem, "InterfaceNumber"),
+				&interface_number) < 0)
+		return -EINVAL;
+
+	pass_through = l_strdup_printf(QMI_WWAN_PASS_THROUGH, ifname);
+	if (stat(pass_through, &st) == -1) {
+		if (errno == ENOENT)
+			no_pass_through = true;
+		else
+			return -errno;
 	}
 
-	data->n_premux = n_premux;
-	data->main_net_ifindex =
-		ofono_modem_get_integer(modem, "NetworkInterfaceIndex");
-	l_strlcpy(data->main_net_name,
-			ofono_modem_get_string(modem, "NetworkInterface"),
-			sizeof(data->main_net_name));
+	data = l_new(struct gobi_data, 1);
+	data->main_net_ifindex = ifindex;
+	l_strlcpy(data->main_net_name, ifname, sizeof(data->main_net_name));
+	data->interface_number = interface_number;
+	data->no_pass_through = no_pass_through;
+	/* See drivers/net/ethernet/qualcomm/rmnet/rmnet_private.h */
+	data->max_aggregation_size = 16384;
+
 	ofono_modem_set_data(modem, data);
 	ofono_modem_set_capabilities(modem, OFONO_MODEM_CAPABILITY_LTE);
 
@@ -221,6 +317,16 @@ static void gobi_remove(struct ofono_modem *modem)
 		data->set_powered_id = 0;
 	}
 
+	if (data->set_mtu_id) {
+		l_netlink_cancel(l_rtnl_get(), data->set_mtu_id);
+		data->set_mtu_id = 0;
+	}
+
+	if (data->rmnet_id) {
+		rmnet_cancel(data->rmnet_id);
+		data->rmnet_id = 0;
+	}
+
 	cleanup_services(data);
 
 	qmi_qmux_device_free(data->device);
@@ -243,18 +349,28 @@ static void add_service_request(struct gobi_data *data,
 	data->service_requests[data->num_service_requests++] = req;
 }
 
-static void shutdown_cb(void *user_data)
+static void __shutdown_device(struct ofono_modem *modem)
 {
-	struct ofono_modem *modem = user_data;
 	struct gobi_data *data = ofono_modem_get_data(modem);
 
-	DBG("");
-
 	data->discover_attempts = 0;
+	memset(&data->service_requests, 0, sizeof(data->service_requests));
+	data->cur_service_request = 0;
+	data->num_service_requests = 0;
+	data->features = 0;
+	data->data_format = WDA_DATA_FORMAT_UNKNOWN;
 
 	qmi_qmux_device_free(data->device);
 	data->device = NULL;
+}
 
+static void shutdown_cb(void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+
+	DBG("");
+
+	__shutdown_device(modem);
 	ofono_modem_set_powered(modem, FALSE);
 }
 
@@ -266,6 +382,14 @@ static void shutdown_device(struct ofono_modem *modem)
 
 	cleanup_services(data);
 
+	if (!l_memeqzero(data->rmnet_interfaces,
+					sizeof(data->rmnet_interfaces))) {
+		rmnet_del_interfaces(data->n_premux, data->rmnet_interfaces);
+		data->n_premux = 0;
+		memset(data->rmnet_interfaces, 0,
+						sizeof(data->rmnet_interfaces));
+	}
+
 	if (qmi_qmux_device_shutdown(data->device, shutdown_cb, modem, NULL) < 0)
 		shutdown_cb(modem);
 }
@@ -367,99 +491,6 @@ error:
 	shutdown_device(modem);
 }
 
-static void setup_qmi_wwan(const char *interface, uint32_t llproto)
-{
-	char raw_ip;
-	char new_raw_ip;
-
-	if (l_sysctl_get_char(&raw_ip, "/sys/class/net/%s/qmi/raw_ip",
-				interface) < 0) {
-		DBG("Couldn't query raw_ip setting");
-		return;
-	}
-
-	if (raw_ip != 'Y' && raw_ip != 'N') {
-		DBG("Unexpected value: %c", raw_ip);
-		return;
-	}
-
-	switch (llproto) {
-	case QMI_WDA_DATA_LINK_PROTOCOL_802_3:
-		new_raw_ip = 'N';
-		break;
-	case QMI_WDA_DATA_LINK_PROTOCOL_RAW_IP:
-		new_raw_ip = 'Y';
-		break;
-	default:
-		DBG("Unknown WDA Link Protocol");
-		return;
-	}
-
-	DBG("raw_ip: %c, want: %c", raw_ip, new_raw_ip);
-
-	if (raw_ip == new_raw_ip)
-		return;
-
-	if (l_sysctl_set_char(new_raw_ip, "/sys/class/net/%s/qmi/raw_ip",
-				interface) < 0)
-		DBG("Fail to set raw_ip to %c", new_raw_ip);
-}
-
-static void get_data_format_cb(struct qmi_result *result, void *user_data)
-{
-	struct ofono_modem *modem = user_data;
-	struct gobi_data *data = ofono_modem_get_data(modem);
-	uint32_t llproto;
-
-	DBG("");
-
-	if (qmi_result_set_error(result, NULL))
-		goto done;
-
-	if (!qmi_result_get_uint32(result, QMI_WDA_LL_PROTOCOL, &llproto))
-		goto done;
-
-	if (data->using_qmi_wwan) {
-		const char *interface =
-			ofono_modem_get_string(modem, "NetworkInterface");
-
-		setup_qmi_wwan(interface, llproto);
-	}
-
-done:
-	if (qmi_service_send(data->dms, QMI_DMS_GET_CAPS, NULL,
-						get_caps_cb, modem, NULL) > 0)
-		return;
-
-	shutdown_device(modem);
-}
-
-static void create_wda_cb(struct qmi_service *service, void *user_data)
-{
-	struct ofono_modem *modem = user_data;
-	struct gobi_data *data = ofono_modem_get_data(modem);
-
-	DBG("");
-
-	if (!service) {
-		DBG("Failed to request WDA service, continue initialization");
-		goto error;
-	}
-
-	data->wda = service;
-
-	if (qmi_service_send(data->wda, QMI_WDA_GET_DATA_FORMAT, NULL,
-				get_data_format_cb, modem, NULL) > 0)
-		return;
-
-error:
-	if (qmi_service_send(data->dms, QMI_DMS_GET_CAPS, NULL,
-						get_caps_cb, modem, NULL) > 0)
-		return;
-
-	shutdown_device(modem);
-}
-
 static void request_service_cb(struct qmi_service *service, void *user_data)
 {
 	struct ofono_modem *modem = user_data;
@@ -476,10 +507,10 @@ static void request_service_cb(struct qmi_service *service, void *user_data)
 
 	data->cur_service_request += 1;
 	if (data->cur_service_request == data->num_service_requests) {
-		DBG("All services requested, proceeding to create WDA");
+		DBG("All services requested, query DMS Capabilities");
 
-		if (qmi_qmux_device_create_client(data->device, QMI_SERVICE_WDA,
-						create_wda_cb, modem, NULL))
+		if (qmi_service_send(data->dms, QMI_DMS_GET_CAPS, NULL,
+						get_caps_cb, modem, NULL) > 0)
 			return;
 
 		goto error;
@@ -496,13 +527,225 @@ error:
 	shutdown_device(modem);
 }
 
+static uint32_t start_service_requests(struct ofono_modem *modem)
+{
+	struct gobi_data *data = ofono_modem_get_data(modem);
+	unsigned int i;
+
+	DBG("");
+
+	for (i = 0; i < (data->n_premux ? data->n_premux : 1); i++) {
+		add_service_request(data, &data->context_services[i].wds_ipv4,
+					QMI_SERVICE_WDS);
+		add_service_request(data, &data->context_services[i].wds_ipv6,
+					QMI_SERVICE_WDS);
+	}
+
+	return qmi_qmux_device_create_client(data->device, QMI_SERVICE_DMS,
+				request_service_cb, modem, NULL);
+}
+
+static void rmnet_get_interfaces_cb(int error, unsigned int n_interfaces,
+					const struct rmnet_ifinfo *interfaces,
+					void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct gobi_data *data = ofono_modem_get_data(modem);
+	unsigned int i;
+
+	DBG("error: %d, n_interfaces: %u", error, n_interfaces);
+	data->rmnet_id = 0;
+
+	if (error)
+		goto error;
+
+	DBG("RMNet interfaces created:");
+
+	for (i = 0; i < n_interfaces; i++)
+		DBG("\t%s[%d], mux_id: %u",
+			interfaces[i].ifname, interfaces[i].ifindex,
+			interfaces[i].mux_id);
+
+	memcpy(data->rmnet_interfaces, interfaces,
+				sizeof(struct rmnet_ifinfo) * n_interfaces);
+	data->n_premux = n_interfaces;
+
+	if (start_service_requests(modem) > 0)
+		return;
+error:
+	shutdown_device(modem);
+}
+
+static void enable_set_mtu_cb(int error, uint16_t type,
+					const void *msg, uint32_t len,
+					void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct gobi_data *data = ofono_modem_get_data(modem);
+
+	DBG("error: %d", error);
+
+	data->set_mtu_id = 0;
+
+	if (error)
+		goto error;
+
+	if (data->data_format != WDA_DATA_FORMAT_802_3) {
+		DBG("Setting QMI WWAN to raw_ip");
+
+		if (qmi_wwan_set_raw_ip(data->main_net_name, 'Y') < 0) {
+			ofono_warn("Unable to set raw_ip");
+			goto error;
+		}
+	}
+
+	if (L_IN_SET(data->data_format, WDA_DATA_FORMAT_RMNET_QMAP5)) {
+		DBG("Setting QMI WWAN to pass_through");
+
+		if (qmi_wwan_set_pass_through(data->main_net_name, 'Y') < 0) {
+			ofono_error("Unable to set pass_through");
+			goto error;
+		}
+
+		data->rmnet_id = rmnet_get_interfaces(data->main_net_ifindex,
+							MAX_CONTEXTS,
+							rmnet_get_interfaces_cb,
+							modem, NULL);
+		if (data->rmnet_id > 0)
+			return;
+
+		ofono_error("Unable to request RMNet interfaces");
+		goto error;
+	}
+
+	if (start_service_requests(modem) > 0)
+		return;
+error:
+	shutdown_device(modem);
+}
+
+/*
+ * Some firmware accepts the Set Data Format command successfully, but doesn't
+ * change anything.  Try to detect and warn if that happens
+ */
+static int compare_data_format(struct gobi_data *data, struct qmi_result *result)
+{
+	struct qmi_wda_data_format expected;
+	struct qmi_wda_data_format actual;
+	int r;
+
+	if (!wda_get_data_format(data, &expected))
+		return -EPROTO;
+
+	r = qmi_wda_parse_data_format(result, &actual);
+	if (r < 0)
+		return -EBADMSG;
+
+	if (expected.ll_protocol != actual.ll_protocol ||
+			expected.ul_aggregation_protocol !=
+			actual.ul_aggregation_protocol ||
+			expected.dl_aggregation_protocol !=
+			actual.dl_aggregation_protocol)
+		return -EBADE;
+
+	return 0;
+}
+
+static void set_data_format_cb(struct qmi_result *result, void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct gobi_data *data = ofono_modem_get_data(modem);
+	struct qmi_endpoint_info endpoint_info = {
+		.endpoint_type = QMI_DATA_ENDPOINT_TYPE_HSUSB,
+		.interface_number = data->interface_number,
+	};
+	struct qmi_wda_data_format format;
+	uint32_t mtu;
+
+	DBG("");
+
+	if (!qmi_result_set_error(result, NULL)) {
+		if (!compare_data_format(data, result))
+			goto done;
+
+		ofono_error("Setting Data Format had no effect");
+	}
+
+	if (data->data_format == WDA_DATA_FORMAT_802_3)
+		goto error;
+
+	DBG("Trying next data format");
+	data->data_format += 1;
+
+	if (!wda_get_data_format(data, &format))
+		goto error;
+
+	if (qmi_wda_set_data_format(data->wda, &endpoint_info, &format,
+					set_data_format_cb, modem, NULL) > 0)
+		return;
+
+	goto error;
+
+done:
+	DBG("Set Data Format succeeded, try to set MTU...");
+
+	if (L_WARN_ON(wda_data_format_to_mtu(data->data_format, &mtu) < 0))
+		goto error;
+
+	data->set_mtu_id = l_rtnl_link_set_mtu(l_rtnl_get(),
+						data->main_net_ifindex, mtu,
+						enable_set_mtu_cb, modem, NULL);
+	if (data->set_mtu_id)
+		return;
+error:
+	shutdown_device(modem);
+}
+
+static void create_wda_cb(struct qmi_service *service, void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct gobi_data *data = ofono_modem_get_data(modem);
+	struct qmi_endpoint_info endpoint_info = {
+		.endpoint_type = QMI_DATA_ENDPOINT_TYPE_HSUSB,
+		.interface_number = data->interface_number,
+	};
+	struct qmi_wda_data_format format;
+
+	DBG("");
+
+	if (!service) {
+		DBG("Failed to request WDA service, assume 802.3");
+
+		if (qmi_qmux_device_create_client(data->device, QMI_SERVICE_DMS,
+					request_service_cb, modem, NULL) > 0)
+			return;
+
+		goto error;
+	}
+
+	data->wda = service;
+
+	if (data->no_pass_through)
+		data->data_format = WDA_DATA_FORMAT_802_3;
+	else
+		data->data_format = WDA_DATA_FORMAT_UNKNOWN + 1;
+
+	if (!wda_get_data_format(data, &format))
+		goto error;
+
+	if (qmi_wda_set_data_format(data->wda, &endpoint_info, &format,
+					set_data_format_cb, modem, NULL) > 0)
+		return;
+error:
+	shutdown_device(modem);
+}
+
 static void discover_cb(void *user_data)
 {
 	struct ofono_modem *modem = user_data;
 	struct gobi_data *data = ofono_modem_get_data(modem);
 	uint16_t major;
 	uint16_t minor;
-	int i;
 
 	DBG("");
 
@@ -552,25 +795,57 @@ static void discover_cb(void *user_data)
 	if (data->features & GOBI_UIM)
 		add_service_request(data, &data->uim, QMI_SERVICE_UIM);
 
-	for (i = 0; i < (data->n_premux ? data->n_premux : 1); i++) {
-		add_service_request(data, &data->context_services[i].wds_ipv4,
-							QMI_SERVICE_WDS);
-		add_service_request(data, &data->context_services[i].wds_ipv6,
-							QMI_SERVICE_WDS);
-	}
-
-	if (qmi_qmux_device_create_client(data->device, QMI_SERVICE_DMS,
-					request_service_cb, modem, NULL) > 0)
+	if (qmi_qmux_device_create_client(data->device, QMI_SERVICE_WDA,
+						create_wda_cb, modem, NULL))
 		return;
+
 error:
 	shutdown_device(modem);
 }
 
+static void init_powered_down_cb(int error, uint16_t type,
+					const void *msg, uint32_t len,
+					void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct gobi_data *data = ofono_modem_get_data(modem);
+	int r;
+
+	DBG("error: %d", error);
+
+	data->set_powered_id = 0;
+
+	if (error)
+		goto error;
+
+	DBG("Setting QMI_WWAN to 802.3 mode");
+
+	/* Must set pass_through first, before toggling raw_ip */
+	if (!data->no_pass_through) {
+		r = qmi_wwan_set_pass_through(data->main_net_name, 'N');
+		if (r < 0) {
+			ofono_warn("Unable to reset pass_through");
+			goto error;
+		}
+	}
+
+	r = qmi_wwan_set_raw_ip(data->main_net_name, 'N');
+	if (r < 0) {
+		ofono_warn("Unable to reset raw_ip");
+		goto error;
+	}
+
+	r = qmi_qmux_device_discover(data->device, discover_cb, modem, NULL);
+	if (!r)
+		return;
+error:
+	__shutdown_device(modem);
+}
+
 static int gobi_enable(struct ofono_modem *modem)
 {
 	struct gobi_data *data = ofono_modem_get_data(modem);
 	const char *device;
-	int r;
 
 	DBG("%p", modem);
 
@@ -589,11 +864,15 @@ static int gobi_enable(struct ofono_modem *modem)
 		qmi_qmux_device_set_io_debug(data->device,
 						gobi_io_debug, "QMI: ");
 
-	r = qmi_qmux_device_discover(data->device, discover_cb, modem, NULL);
-	if (!r)
+	data->set_powered_id =
+		l_rtnl_set_powered(l_rtnl_get(), data->main_net_ifindex,
+					false, init_powered_down_cb,
+					modem, NULL);
+	if (data->set_powered_id)
 		return -EINPROGRESS;
 
-	return r;
+	__shutdown_device(modem);
+	return -EIO;
 }
 
 static void power_disable_cb(struct qmi_result *result, void *user_data)
@@ -670,10 +949,6 @@ static void powered_up_cb(int error, uint16_t type,
 	if (!param)
 		goto error;
 
-	if (data->using_qmi_wwan_q)
-		l_sysctl_set_u32(1, "/sys/class/net/%s/link_state",
-					data->main_net_name);
-
 	cb_data_ref(cbd);
 
 	if (qmi_service_send(data->dms, QMI_DMS_SET_OPER_MODE, param,
@@ -707,10 +982,6 @@ static void powered_down_cb(int error, uint16_t type,
 	if (!param)
 		goto error;
 
-	if (data->using_qmi_wwan_q)
-		l_sysctl_set_u32(0, "/sys/class/net/%s/link_state",
-					data->main_net_name);
-
 	cb_data_ref(cbd);
 
 	if (qmi_service_send(data->dms, QMI_DMS_SET_OPER_MODE, param,
@@ -732,7 +1003,7 @@ static void gobi_set_online(struct ofono_modem *modem, ofono_bool_t online,
 	l_netlink_command_func_t powered_cb;
 
 	DBG("%p %s using_mux: %s", modem, online ? "online" : "offline",
-		data->using_mux ? "yes" : "no");
+		data->n_premux ? "yes" : "no");
 
 	cbd->user = data;
 
@@ -741,7 +1012,7 @@ static void gobi_set_online(struct ofono_modem *modem, ofono_bool_t online,
 	else
 		powered_cb = powered_down_cb;
 
-	if (!data->using_mux) {
+	if (!data->n_premux) {
 		powered_cb(0, 0, NULL, 0, cbd);
 		cb_data_unref(cbd);
 		return;
@@ -788,8 +1059,6 @@ static void gobi_setup_gprs(struct ofono_modem *modem)
 	struct gobi_data *data = ofono_modem_get_data(modem);
 	struct ofono_gprs *gprs;
 	struct ofono_gprs_context *gc;
-	const char *interface;
-	char buf[256];
 	int i;
 
 	gprs = ofono_gprs_create(modem, 0, "qmimodem",
@@ -806,8 +1075,6 @@ static void gobi_setup_gprs(struct ofono_modem *modem)
 		struct qmi_service *ipv4 = data->context_services[0].wds_ipv4;
 		struct qmi_service *ipv6 = data->context_services[0].wds_ipv6;
 
-		interface = ofono_modem_get_string(modem, "NetworkInterface");
-
 		gc = ofono_gprs_context_create(modem, 0, "qmimodem", -1,
 						qmi_service_clone(ipv4),
 						qmi_service_clone(ipv6));
@@ -818,26 +1085,18 @@ static void gobi_setup_gprs(struct ofono_modem *modem)
 		}
 
 		ofono_gprs_add_context(gprs, gc);
-		ofono_gprs_context_set_interface(gc, interface);
+		ofono_gprs_context_set_interface(gc, data->main_net_name);
 
 		return;
 	}
 
-	data->using_mux = true;
-
-	data->max_aggregation_size =
-		ofono_modem_get_integer(modem, "MaxAggregationSize");
-	DBG("max_aggregation_size: %u", data->max_aggregation_size);
-
 	for (i = 0; i < data->n_premux; i++) {
 		struct qmi_service *ipv4 = data->context_services[i].wds_ipv4;
 		struct qmi_service *ipv6 = data->context_services[i].wds_ipv6;
-		int mux_id;
+		struct rmnet_ifinfo *ifinfo = data->rmnet_interfaces + i;
 
-		sprintf(buf, "PremuxInterface%dMuxId", i + 1);
-		mux_id = ofono_modem_get_integer(modem, buf);
-
-		gc = ofono_gprs_context_create(modem, 0, "qmimodem", mux_id,
+		gc = ofono_gprs_context_create(modem, 0, "qmimodem",
+						ifinfo->mux_id,
 						qmi_service_clone(ipv4),
 						qmi_service_clone(ipv6));
 
@@ -847,11 +1106,8 @@ static void gobi_setup_gprs(struct ofono_modem *modem)
 			continue;
 		}
 
-		sprintf(buf, "PremuxInterface%d", i + 1);
-		interface = ofono_modem_get_string(modem, buf);
-
 		ofono_gprs_add_context(gprs, gc);
-		ofono_gprs_context_set_interface(gc, interface);
+		ofono_gprs_context_set_interface(gc, ifinfo->ifname);
 	}
 }
 
